@@ -20,7 +20,7 @@ class DolphinMainWindowTest : public QObject
 {
     Q_OBJECT
 
-private slots:
+private Q_SLOTS:
     void initTestCase();
     void init();
     void testClosingTabsWithSearchBoxVisible();
