@@ -18,6 +18,7 @@
 #include "kitemviews/kitemlistcontroller.h"
 #include "kitemviews/kitemlistheader.h"
 #include "kitemviews/kitemlistselectionmanager.h"
+#include "kitemviews/private/kitemlistroleeditor.h"
 #include "versioncontrol/versioncontrolobserver.h"
 #include "viewproperties.h"
 #include "views/tooltips/tooltipmanager.h"
@@ -87,7 +88,8 @@ DolphinView::DolphinView(const QUrl& url, QWidget* parent) :
     m_markFirstNewlySelectedItemAsCurrent(false),
     m_versionControlObserver(nullptr),
     m_twoClicksRenamingTimer(nullptr),
-    m_placeholderLabel(nullptr)
+    m_placeholderLabel(nullptr),
+    m_showLoadingPlaceholderTimer(nullptr)
 {
     m_topLayout = new QVBoxLayout(this);
     m_topLayout->setSpacing(0);
@@ -125,6 +127,11 @@ DolphinView::DolphinView(const QUrl& url, QWidget* parent) :
     connect(m_container->horizontalScrollBar(), &QScrollBar::valueChanged, this, [=] { hideToolTip(); });
     connect(m_container->verticalScrollBar(), &QScrollBar::valueChanged, this, [=] { hideToolTip(); });
 
+    m_showLoadingPlaceholderTimer = new QTimer(this);
+    m_showLoadingPlaceholderTimer->setInterval(500);
+    m_showLoadingPlaceholderTimer->setSingleShot(true);
+    connect(m_showLoadingPlaceholderTimer, &QTimer::timeout, this, &DolphinView::showLoadingPlaceholder);
+
     // Show some placeholder text for empty folders
     // This is made using a heavily-modified QLabel rather than a KTitleWidget
     // because KTitleWidget can't be told to turn off mouse-selectable text
@@ -178,6 +185,7 @@ DolphinView::DolphinView(const QUrl& url, QWidget* parent) :
     connect(m_model, &KFileItemModel::errorMessage,           this, &DolphinView::errorMessage);
     connect(m_model, &KFileItemModel::directoryRedirection, this, &DolphinView::slotDirectoryRedirection);
     connect(m_model, &KFileItemModel::urlIsFileError,            this, &DolphinView::urlIsFileError);
+    connect(m_model, &KFileItemModel::fileItemsChanged,            this, &DolphinView::fileItemsChanged);
 
     connect(this, &DolphinView::itemCountChanged,
             this, &DolphinView::updatePlaceholderLabel);
@@ -674,12 +682,21 @@ void DolphinView::renameSelectedItems()
 
     if (items.count() == 1 && GeneralSettings::renameInline()) {
         const int index = m_model->index(items.first());
-        m_view->editRole(index, "text");
 
-        hideToolTip();
+        QMetaObject::Connection * const connection = new QMetaObject::Connection;
+        *connection = connect(m_view, &KItemListView::scrollingStopped, this, [=](){
+            QObject::disconnect(*connection);
+            delete connection;
+
+            m_view->editRole(index, "text");
+
+            hideToolTip();
+
+            connect(m_view, &DolphinItemListView::roleEditingFinished,
+                    this, &DolphinView::slotRoleEditingFinished);
+        });
+        m_view->scrollToItem(index);
 
-        connect(m_view, &DolphinItemListView::roleEditingFinished,
-                this, &DolphinView::slotRoleEditingFinished);
     } else {
         KIO::RenameFileDialog* dialog = new KIO::RenameFileDialog(items, this);
         connect(dialog, &KIO::RenameFileDialog::renamingFinished,
@@ -1736,13 +1753,15 @@ void DolphinView::slotRoleEditingFinished(int index, const QByteArray& role, con
     disconnect(m_view, &DolphinItemListView::roleEditingFinished,
                this, &DolphinView::slotRoleEditingFinished);
 
-    if (index < 0 || index >= m_model->count()) {
+    const KFileItemList items = selectedItems();
+    if (items.count() != 1) {
         return;
     }
 
     if (role == "text") {
-        const KFileItem oldItem = m_model->fileItem(index);
-        const QString newName = value.toString();
+        const KFileItem oldItem = items.first();
+        const EditResult retVal = value.value<EditResult>();
+        const QString newName = retVal.newName;
         if (!newName.isEmpty() && newName != oldItem.text() && newName != QLatin1Char('.') && newName != QLatin1String("..")) {
             const QUrl oldUrl = oldItem.url();
 
@@ -1773,14 +1792,14 @@ void DolphinView::slotRoleEditingFinished(int index, const QByteArray& role, con
 #endif
 
             const bool newNameExistsAlready = (m_model->index(newUrl) >= 0);
-            if (!newNameExistsAlready) {
+            if (!newNameExistsAlready && m_model->index(oldUrl) == index) {
                 // Only change the data in the model if no item with the new name
                 // is in the model yet. If there is an item with the new name
                 // already, calling KIO::CopyJob will open a dialog
                 // asking for a new name, and KFileItemModel will update the
                 // data when the dir lister signals that the file name has changed.
                 QHash<QByteArray, QVariant> data;
-                data.insert(role, value);
+                data.insert(role, retVal.newName);
                 m_model->setData(index, data);
             }
 
@@ -1797,6 +1816,13 @@ void DolphinView::slotRoleEditingFinished(int index, const QByteArray& role, con
                 connect(job, &KJob::result, this, &DolphinView::slotRenamingResult);
             }
         }
+        if (retVal.direction != EditDone) {
+            const short indexShift = retVal.direction == EditNext ? 1 : -1;
+            m_container->controller()->selectionManager()->setSelected(index, 1, KItemListSelectionManager::Deselect);
+            m_container->controller()->selectionManager()->setSelected(index + indexShift, 1,
+                    KItemListSelectionManager::Select);
+            renameSelectedItems();
+        }
     }
 }
 
@@ -2044,13 +2070,26 @@ void DolphinView::slotSwipeUp()
     Q_EMIT goUpRequested();
 }
 
+void DolphinView::showLoadingPlaceholder()
+{
+    m_placeholderLabel->setText(i18n("Loading..."));
+    m_placeholderLabel->setVisible(true);
+}
+
 void DolphinView::updatePlaceholderLabel()
 {
-    if (m_loading || itemsCount() > 0) {
+    m_showLoadingPlaceholderTimer->stop();
+    if (itemsCount() > 0) {
         m_placeholderLabel->setVisible(false);
         return;
     }
 
+    if (m_loading) {
+        m_placeholderLabel->setVisible(false);
+        m_showLoadingPlaceholderTimer->start();
+        return;
+    }
+
     if (!nameFilter().isEmpty()) {
         m_placeholderLabel->setText(i18n("No items matching the filter"));
     } else if (m_url.scheme() == QLatin1String("baloosearch") || m_url.scheme() == QLatin1String("filenamesearch")) {
@@ -2065,7 +2104,7 @@ void DolphinView::updatePlaceholderLabel()
         m_placeholderLabel->setText(i18n("No shared folders found"));
     } else if (m_url.scheme() == QLatin1String("network")) {
         m_placeholderLabel->setText(i18n("No relevant network resources found"));
-    } else if (m_url.scheme() == QLatin1String("mtp")) {
+    } else if (m_url.scheme() == QLatin1String("mtp") && m_url.path() == QLatin1String("/")) {
         m_placeholderLabel->setText(i18n("No MTP-compatible devices found"));
     } else if (m_url.scheme() == QLatin1String("bluetooth")) {
         m_placeholderLabel->setText(i18n("No Bluetooth devices found"));
